--[[
	BaleWrapperExtension.lua
	
	@author: 			[LSFM] BayernGamers
	@contributor: 		Ifko[nator]
	@date: 				11.11.2024
	@version: 			3.0
	
	Changelog:			v1.0 @25.12.2018 - initial implementation in FS 19
						------------------------------------------------------------------------
						v1.5 @10.08.2020 - add support for the stationary baler from Farmer_Andy
						------------------------------------------------------------------------
						v2.0 @30.03.2022 - convert to FS 22
						------------------------------------------------------------------------
						v2.1 @19.03.2023 - fix for Patch 1.9.0.0 and goeweil DLC
						------------------------------------------------------------------------
						v3.0 @11.11.2024 - convert to FS25
]]

source(Utils.getFilename("scripts/utils/LoggingUtil.lua", g_currentModDirectory))
source(Utils.getFilename("scripts/SetDropModeEvent.lua", g_currentModDirectory))
source(Utils.getFilename("scripts/ToggleBaleWrapperEvent.lua", g_currentModDirectory))

local log = LoggingUtil

BaleWrapperExtension = {}
BaleWrapperExtension.modName = g_currentModName

BaleWrapperExtension.DROP_MODE_MANUAL = 1
BaleWrapperExtension.DROP_MODE_AUTOMATIC = 2
BaleWrapperExtension.DROP_MODE_COLLECT = 3
BaleWrapperExtension.DROP_MODE_MAX_STATE = 2
BaleWrapperExtension.DROP_MODE_BALER_MAX_STATE = 3
BaleWrapperExtension.FORCE_DROP_BALE = false

function BaleWrapperExtension.initSpecialization()
	local schemaSavegame = Vehicle.xmlSchemaSavegame

	schemaSavegame:register(XMLValueType.INT, "vehicles.vehicle(?)." .. BaleWrapperExtension.modName .. ".baleWrapperExtension#currentDropModeState", "Current drop mode.", BaleWrapperExtension.DROP_MODE_MANUAL)
	schemaSavegame:register(XMLValueType.BOOL, "vehicles.vehicle(?)." .. BaleWrapperExtension.modName .. ".baleWrapperExtension#baleWrapperIsActive", "Is bale wrapper active.", true)
end

function BaleWrapperExtension.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(BaleWrapper, specializations)
end

function BaleWrapperExtension.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad",
		"onUpdate",
		"saveToXMLFile",
		"onWriteStream",
		"onReadStream",
		"onRegisterActionEvents"
	}
	
	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, BaleWrapperExtension)
	end
end

function BaleWrapperExtension.registerFunctions(vehicleType)
	local functionNames = {
		"setDropMode",
		"toggleBaleWrapper"
	}
	
	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerFunction(vehicleType, functionName, BaleWrapperExtension[functionName])
	end
end

function BaleWrapperExtension:onLoad(savegame)
	self.spec_baleWrapperExtension = {}
	local spec_baleWrapperExtension = self.spec_baleWrapperExtension
	local spec_baler = self.spec_baler

	spec_baleWrapperExtension.foundGoeweilDLC = self.xmlFile.filename:find("data/vehicles/goeweil/varioMaster/varioMaster.xml")

	--## deactivate default button for switching between automatic and manual unload for balers with bale wrappers
	self.spec_baleWrapper.toggleableAutomaticDrop = false
	
	if savegame ~= nil and not savegame.resetVehicles then
		spec_baleWrapperExtension.currentDropModeState = savegame.xmlFile:getValue(savegame.key .. "." .. BaleWrapperExtension.modName .. ".baleWrapperExtension#currentDropModeState", BaleWrapperExtension.DROP_MODE_MANUAL)
		
		if spec_baler ~= nil and not spec_baleWrapperExtension.foundGoeweilDLC then	
			spec_baleWrapperExtension.baleWrapperIsActive = savegame.xmlFile:getValue(savegame.key .. "." .. BaleWrapperExtension.modName .. ".baleWrapperExtension#baleWrapperIsActive", true)
		end
	else
		spec_baleWrapperExtension.currentDropModeState = BaleWrapperExtension.DROP_MODE_MANUAL
		
		if spec_baler ~= nil and not spec_baleWrapperExtension.foundGoeweilDLC then
			spec_baleWrapperExtension.baleWrapperIsActive = true
		end
	end
	
	self:setDropMode(spec_baleWrapperExtension.currentDropModeState, false)

	spec_baleWrapperExtension.l10NTexts = {}
	spec_baleWrapperExtension.maxDropState = BaleWrapperExtension.DROP_MODE_MAX_STATE
	
	if spec_baler ~= nil and not spec_baleWrapperExtension.foundGoeweilDLC then
		self:toggleBaleWrapper(spec_baleWrapperExtension.baleWrapperIsActive)
		
		self.doStateChange = Utils.overwrittenFunction(self.doStateChange, BaleWrapperExtension.doStateChange)
		self.pickupWrapperBale = Utils.overwrittenFunction(self.pickupWrapperBale, BaleWrapperExtension.pickupWrapperBale)

		local l10NTexts = {
			"START_BALE_WRAPPER",
			"STOP_BALE_WRAPPER",
			"DROP_MODE_COLLECT",
		}
	
		for _, l10NText in pairs(l10NTexts) do
			spec_baleWrapperExtension.l10NTexts[l10NText] = g_i18n:getText(l10NText, BaleWrapperExtension.modName)
		end

		--## enable the collect mode for balers with bale wrappers
		spec_baleWrapperExtension.maxDropState = BaleWrapperExtension.DROP_MODE_BALER_MAX_STATE
	end

	local l10NTexts = {
		"DROP_MODE_MANUALLY",
		"DROP_MODE_AUTOMATIC",
		"CURRENT_DROP_MODE"
	}

	for _, l10NText in pairs(l10NTexts) do
		spec_baleWrapperExtension.l10NTexts[l10NText] = g_i18n:getText(l10NText, BaleWrapperExtension.modName)
	end
end

function BaleWrapperExtension:setDropMode(currentDropModeState, noEventSend)
    local spec_baleWrapperExtension = self.spec_baleWrapperExtension

	if currentDropModeState ~= spec_baleWrapperExtension.currentDropModeState then
		spec_baleWrapperExtension.currentDropModeState = currentDropModeState
		
		 if not noEventSend then
			if g_server ~= nil then
				g_server:broadcastEvent(SetDropModeEvent.new(self, currentDropModeState), nil, nil, self)
			else
				g_client:getServerConnection():sendEvent(SetDropModeEvent.new(self, currentDropModeState))
			end
		end
	end
end

function BaleWrapperExtension:onWriteStream(streamId, connection)
	if not connection:getIsServer() then 
		local spec_baleWrapperExtension = self.spec_baleWrapperExtension
		local spec_baler = self.spec_baler
		
		streamWriteUIntN(streamId, spec_baleWrapperExtension.currentDropModeState, spec_baleWrapperExtension.maxDropState)

		if spec_baler ~= nil and not spec_baleWrapperExtension.foundGoeweilDLC then	
			streamWriteBool(streamId, spec_baleWrapperExtension.baleWrapperIsActive)
		end
	end
end

function BaleWrapperExtension:onReadStream(streamId, connection)
	if connection:getIsServer() then
		local spec_baleWrapperExtension = self.spec_baleWrapperExtension
		local spec_baler = self.spec_baler
		
		spec_baleWrapperExtension.currentDropModeState = streamReadUIntN(streamId, spec_baleWrapperExtension.maxDropState)
		
		if spec_baler ~= nil and not spec_baleWrapperExtension.foundGoeweilDLC then
			spec_baleWrapperExtension.baleWrapperIsActive = streamReadBool(streamId)
		end
	end
end

function BaleWrapperExtension:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
	if self.isClient then
        local spec_baleWrapperExtension = self.spec_baleWrapperExtension
        
		spec_baleWrapperExtension.actionEvents = {}
		self:clearActionEventsTable(spec_baleWrapperExtension.actionEvents)
        
		if self:getIsActiveForInput(true, false) then
            local newFunctions = {
				"CHANGE_DROP_MODE_BUTTON",
				"TOGGLE_BALE_WRAPPER_BUTTON"
			}
			
			for _, newFunction in ipairs(newFunctions) do
				local _, actionEventId = self:addPoweredActionEvent(spec_baleWrapperExtension.actionEvents, InputAction[newFunction], self, BaleWrapperExtension.actionEventChangeDropeMode, false, true, false, true, nil)
			
				g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_HIGH)
				g_inputBinding:setActionEventActive(actionEventId, false)
			end
		end
	end
end

function BaleWrapperExtension.actionEventChangeDropeMode(self, actionName, inputValue, callbackState, isAnalog)
	local spec_baleWrapperExtension = self.spec_baleWrapperExtension
	local spec_baler = self.spec_baler
	
	if actionName == "CHANGE_DROP_MODE_BUTTON" then
		if spec_baleWrapperExtension.currentDropModeState < spec_baleWrapperExtension.maxDropState then
			self:setDropMode(spec_baleWrapperExtension.currentDropModeState + 1)
		else
			self:setDropMode(BaleWrapperExtension.DROP_MODE_MANUAL)
		end
	else
		if spec_baler ~= nil then	
			self:toggleBaleWrapper(not spec_baleWrapperExtension.baleWrapperIsActive)
		end
	end
end

function BaleWrapperExtension:doStateChange(superFunc, id, nearestBaleServerId)
    local spec_baleWrapperExtension = self.spec_baleWrapperExtension
	local spec_baleWrapper = self.spec_baleWrapper
	
	if id == BaleWrapper.CHANGE_WRAPPING_START and not spec_baleWrapperExtension.baleWrapperIsActive and not spec_baleWrapperExtension.foundGoeweilDLC then
        spec_baleWrapper.baleWrapperState = BaleWrapper.STATE_WRAPPER_FINSIHED
        
		return
    end

	if id == BaleWrapper.CHANGE_WRAPPING_START or spec_baleWrapper.baleWrapperState ~= BaleWrapper.STATE_WRAPPER_FINSIHED and id == BaleWrapper.CHANGE_WRAPPER_START_DROP_BALE then
		local bale = NetworkUtil.getObject(spec_baleWrapper.currentWrapper.currentBale)
		local baleType = spec_baleWrapper.currentWrapper.allowedBaleTypes[spec_baleWrapper.currentBaleTypeIndex]

		if not bale:getSupportsWrapping() or baleType.skipWrapping or bale.wrappingState == 1 then
			if self.isServer then
				spec_baleWrapper.setWrappingStateFinished = true
			end

			return
		end
	end

	if id == BaleWrapper.CHANGE_GRAB_BALE then
		local bale = NetworkUtil.getObject(nearestBaleServerId)
		spec_baleWrapper.baleGrabber.currentBale = nearestBaleServerId

		if bale ~= nil then
			local x, y, z = localToLocal(bale.nodeId, getParent(spec_baleWrapper.baleGrabber.grabNode), 0, 0, 0)

			setTranslation(spec_baleWrapper.baleGrabber.grabNode, x, y, z)
			bale:mountKinematic(self, spec_baleWrapper.baleGrabber.grabNode, 0, 0, 0, 0, 0, 0)
			bale:setCanBeSold(false)

			spec_baleWrapper.baleToMount = nil

			self:playMoveToWrapper(bale)
		else
			spec_baleWrapper.baleToMount = {
				serverId = nearestBaleServerId,
				linkNode = spec_baleWrapper.baleGrabber.grabNode,
				trans = {
					0,
					0,
					0
				},
				rot = {
					0,
					0,
					0
				}
			}
		end

		spec_baleWrapper.baleWrapperState = BaleWrapper.STATE_MOVING_BALE_TO_WRAPPER
	elseif id == BaleWrapper.CHANGE_DROP_BALE_AT_GRABBER then
		local attachNode = spec_baleWrapper.currentWrapper.baleNode
		local bale = NetworkUtil.getObject(spec_baleWrapper.baleGrabber.currentBale)

		if bale ~= nil then
			bale:mountKinematic(self, attachNode, 0, 0, 0, 0, 0, 0)
			bale:setCanBeSold(false)

			spec_baleWrapper.baleToMount = nil
		else
			spec_baleWrapper.baleToMount = {
				serverId = spec_baleWrapper.baleGrabber.currentBale,
				linkNode = attachNode,
				trans = {
					0,
					0,
					0
				},
				rot = {
					0,
					0,
					0
				}
			}
		end

		self:updateWrapNodes(true, false, 0)

		spec_baleWrapper.currentWrapper.currentBale = spec_baleWrapper.baleGrabber.currentBale
		spec_baleWrapper.baleGrabber.currentBale = nil

		if spec_baleWrapper.currentWrapper.animations.moveToWrapper.animName ~= nil and spec_baleWrapper.currentWrapper.animations.moveToWrapper.reverseAfterMove then
			self:playAnimation(spec_baleWrapper.currentWrapper.animations.moveToWrapper.animName, -spec_baleWrapper.currentWrapper.animations.moveToWrapper.animSpeed, nil, true)
		end

		spec_baleWrapper.baleWrapperState = BaleWrapper.STATE_MOVING_GRABBER_TO_WORK
	elseif id == BaleWrapper.CHANGE_WRAPPING_START then
		spec_baleWrapper.baleWrapperState = BaleWrapper.STATE_WRAPPER_WRAPPING_BALE

		if self.isClient then
			g_soundManager:playSamples(spec_baleWrapper.currentWrapper.samples.start)
			g_soundManager:playSamples(spec_baleWrapper.currentWrapper.samples.wrap, 0, spec_baleWrapper.currentWrapper.samples.start[1])
		end

		if spec_baleWrapper.currentWrapper.animations.wrapBale.animName ~= nil then
			self:playAnimation(spec_baleWrapper.currentWrapper.animations.wrapBale.animName, spec_baleWrapper.currentWrapper.animations.wrapBale.animSpeed, nil, true)
		end
	elseif id == BaleWrapper.CHANGE_WRAPPING_BALE_FINSIHED then
		if self.isClient then
			g_soundManager:stopSamples(spec_baleWrapper.currentWrapper.samples.wrap)
			g_soundManager:stopSamples(spec_baleWrapper.currentWrapper.samples.stop)

			if spec_baleWrapper.currentWrapper.wrappingSoundEndTime == 1 then
				g_soundManager:playSamples(spec_baleWrapper.currentWrapper.samples.stop)
			end

			g_soundManager:stopSamples(spec_baleWrapper.currentWrapper.samples.start)
		end

		self:updateWrappingState(1, true)

		spec_baleWrapper.baleWrapperState = BaleWrapper.STATE_WRAPPER_FINSIHED
		local bale = NetworkUtil.getObject(spec_baleWrapper.currentWrapper.currentBale)
		local baleType = spec_baleWrapper.currentWrapper.allowedBaleTypes[spec_baleWrapper.currentBaleTypeIndex]
		local skippedWrapping = not bale:getSupportsWrapping() or baleType.skipWrapping

		if skippedWrapping or bale.wrappingState == 1 then
			self:updateWrappingState(0, true)
		end

		if not skippedWrapping then
			local animation = spec_baleWrapper.currentWrapper.animations.resetWrapping

			if animation.animName ~= nil then
				self:playAnimation(animation.animName, animation.animSpeed, nil, true)
			end
		end
	elseif id == BaleWrapper.CHANGE_WRAPPER_START_DROP_BALE then
		self:updateWrapNodes(false, false, 0)

		if spec_baleWrapper.currentWrapper.animations.dropFromWrapper.animName ~= nil then
			self:playAnimation(spec_baleWrapper.currentWrapper.animations.dropFromWrapper.animName, spec_baleWrapper.currentWrapper.animations.dropFromWrapper.animSpeed, nil, true)
		end

		spec_baleWrapper.baleWrapperState = BaleWrapper.STATE_WRAPPER_DROPPING_BALE
	elseif id == BaleWrapper.CHANGE_WRAPPER_BALE_DROPPED then
		local bale = NetworkUtil.getObject(spec_baleWrapper.currentWrapper.currentBale)

		if bale ~= nil then
			bale:unmountKinematic()
			bale:setCanBeSold(true)

			local baleType = spec_baleWrapper.currentWrapper.allowedBaleTypes[spec_baleWrapper.currentBaleTypeIndex]

			if bale:getSupportsWrapping() and not baleType.skipWrapping and spec_baleWrapperExtension.baleWrapperIsActive then
				local stats = g_currentMission:farmStats(self:getOwnerFarmId())
				local total = stats:updateStats("wrappedBales", 1)

				g_achievementManager:tryUnlock("WrappedBales", total)

				if bale.wrappingState < 1 then
					bale:setWrappingState(1)
				end
			end
		end

		spec_baleWrapper.lastDroppedBale = bale
		spec_baleWrapper.currentWrapper.currentBale = nil
		spec_baleWrapper.currentWrapper.currentTime = 0

		if spec_baleWrapper.currentWrapper.animations.resetAfterDrop.animName ~= nil then
			local dropAnimationName = spec_baleWrapper.currentWrapper.animations.dropFromWrapper.animName

			if self:getIsAnimationPlaying(dropAnimationName) then
				self:stopAnimation(dropAnimationName, true)
				self:setAnimationTime(dropAnimationName, 1, true, false)
			end

			self:playAnimation(spec_baleWrapper.currentWrapper.animations.resetAfterDrop.animName, spec_baleWrapper.currentWrapper.animations.resetAfterDrop.animSpeed, nil, true)
		end

		self:setBaleWrapperType(spec_baleWrapper.currentWrapper == spec_baleWrapper.roundBaleWrapper, spec_baleWrapper.currentBaleTypeIndex)

		spec_baleWrapper.baleWrapperState = BaleWrapper.STATE_WRAPPER_RESETTING_PLATFORM
	elseif id == BaleWrapper.CHANGE_WRAPPER_PLATFORM_RESET then
		self:updateWrappingState(0)
		self:updateWrapNodes(false, true, 0)

		spec_baleWrapper.baleWrapperState = BaleWrapper.STATE_NONE
	elseif id == BaleWrapper.CHANGE_BUTTON_EMPTY then
		assert(self.isServer)

		if spec_baleWrapper.baleWrapperState == BaleWrapper.STATE_WRAPPER_FINSIHED then
			g_server:broadcastEvent(BaleWrapperStateEvent.new(self, BaleWrapper.CHANGE_WRAPPER_START_DROP_BALE), true, nil, self)
		end
	end
end

function BaleWrapperExtension:pickupWrapperBale(superFunc, bale, baleType)
    local spec_baleWrapperExtension = self.spec_baleWrapperExtension
	
	if not spec_baleWrapperExtension.baleWrapperIsActive and not spec_baleWrapperExtension.foundGoeweilDLC then 
        g_server:broadcastEvent(BaleWrapperStateEvent.new(self, BaleWrapper.CHANGE_GRAB_BALE, NetworkUtil.getObjectId(bale)), true, nil, self)
    else
        superFunc(self, bale, baleType)
    end
end

function BaleWrapperExtension:toggleBaleWrapper(baleWrapperIsActive, noEventSend)
	local spec_baleWrapperExtension = self.spec_baleWrapperExtension
	
	if baleWrapperIsActive ~= spec_baleWrapperExtension.baleWrapperIsActive then
		if not noEventSend then
			if g_server ~= nil then
				g_server:broadcastEvent(ToggleBaleWrapperEvent.new(self, baleWrapperIsActive), nil, nil, self)
			else
				g_client:getServerConnection():sendEvent(ToggleBaleWrapperEvent.new(self, baleWrapperIsActive))
			end
		end
		
		spec_baleWrapperExtension.baleWrapperIsActive = baleWrapperIsActive
	end
end

function BaleWrapperExtension:saveToXMLFile(xmlFile, key)
	local spec_baleWrapperExtension = self.spec_baleWrapperExtension
	local spec_baler = self.spec_baler
	
	xmlFile:setValue(key .. "#currentDropModeState", spec_baleWrapperExtension.currentDropModeState)
	
	if spec_baler ~= nil and not spec_baleWrapperExtension.foundGoeweilDLC then	
		xmlFile:setValue(key .. "#baleWrapperIsActive", spec_baleWrapperExtension.baleWrapperIsActive)
	end
end

function BaleWrapperExtension:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	if self:getIsActiveForInput(true, false) or string.find(self.configFileName, "stationaryBalerSRBW4K") then
		local spec_baleWrapperExtension = self.spec_baleWrapperExtension
		local spec_baleWrapper = self.spec_baleWrapper
		local spec_baler = self.spec_baler
		
		if spec_baler ~= nil and not spec_baleWrapperExtension.foundGoeweilDLC then
			if spec_baleWrapper.baleToLoad == nil then
				if spec_baleWrapper.baleWrapperState < BaleWrapper.STATE_MOVING_BALE_TO_WRAPPER or spec_baleWrapper.baleWrapperState >= BaleWrapper.STATE_WRAPPER_WRAPPING_BALE then
					if spec_baleWrapperExtension.baleWrapperIsActive and self.isServer then
						local bale = NetworkUtil.getObject(spec_baleWrapper.currentWrapper.currentBale)
						
						if bale ~= nil and bale.wrappingState == 0 and spec_baleWrapper.baleWrapperState == BaleWrapper.STATE_WRAPPER_FINSIHED then
							self:playMoveToWrapper(bale)
						end
					end
				end
			end
			
			local toggleBaleWrapperButton = spec_baleWrapperExtension.actionEvents[InputAction.TOGGLE_BALE_WRAPPER_BUTTON]

			if toggleBaleWrapperButton ~= nil then
				local currentText = spec_baleWrapperExtension.l10NTexts.START_BALE_WRAPPER
				
				if spec_baleWrapperExtension.baleWrapperIsActive then
					currentText = spec_baleWrapperExtension.l10NTexts.STOP_BALE_WRAPPER
				end

				local allowToggleBaleWrapper = spec_baleWrapper.currentWrapper.currentBale == nil
				local currentFillType = g_fillTypeManager:getFillTypeByIndex(self:getFillUnitFillType(spec_baler.fillUnitIndex)).name
				
				if currentFillType ~= nil and currentFillType ~= "UNKNOWN" then
					allowToggleBaleWrapper = allowToggleBaleWrapper and currentFillType == "GRASS_WINDROW"
				end
				
				g_inputBinding:setActionEventActive(toggleBaleWrapperButton.actionEventId, allowToggleBaleWrapper)
				g_inputBinding:setActionEventTextVisibility(toggleBaleWrapperButton.actionEventId, allowToggleBaleWrapper)
				g_inputBinding:setActionEventText(toggleBaleWrapperButton.actionEventId, currentText)
			end
		end
		
		if not spec_baleWrapperExtension.foundGoeweilDLC then
			local changeDropModeButton = spec_baleWrapperExtension.actionEvents[InputAction.CHANGE_DROP_MODE_BUTTON]

			if changeDropModeButton ~= nil then
				local currentText = spec_baleWrapperExtension.l10NTexts.DROP_MODE_MANUALLY

				g_inputBinding:setActionEventActive(changeDropModeButton.actionEventId, true)

				if spec_baleWrapperExtension.currentDropModeState == BaleWrapperExtension.DROP_MODE_AUTOMATIC then
					currentText = spec_baleWrapperExtension.l10NTexts.DROP_MODE_AUTOMATIC
				elseif spec_baleWrapperExtension.currentDropModeState == BaleWrapperExtension.DROP_MODE_COLLECT then
					currentText = spec_baleWrapperExtension.l10NTexts.DROP_MODE_COLLECT
				end

				g_inputBinding:setActionEventText(changeDropModeButton.actionEventId, string.gsub(spec_baleWrapperExtension.l10NTexts.CURRENT_DROP_MODE, "currentDropMode", currentText))
			end

			if spec_baleWrapper.baleWrapperState == BaleWrapper.STATE_WRAPPER_FINSIHED and spec_baleWrapperExtension.currentDropModeState > BaleWrapperExtension.DROP_MODE_MANUAL then
				local allowDropBale = false

				if spec_baleWrapperExtension.currentDropModeState == BaleWrapperExtension.DROP_MODE_AUTOMATIC then
					allowDropBale = true
				elseif BaleWrapperExtension.FORCE_DROP_BALE then
					allowDropBale = true

					BaleWrapperExtension.FORCE_DROP_BALE = false
				elseif spec_baleWrapperExtension.currentDropModeState == BaleWrapperExtension.DROP_MODE_COLLECT then
					if self.spec_fillUnit:getFillUnitFillLevel(1) == self.spec_fillUnit:getFillUnitCapacity(1) then
						allowDropBale = true

						BaleWrapperExtension.FORCE_DROP_BALE = true
					end
				end

				if self.isClient and allowDropBale then
					g_client:getServerConnection():sendEvent(BaleWrapperStateEvent.new(self, BaleWrapper.CHANGE_BUTTON_EMPTY))
				end
			end
		end
	end
end